unit SDLScene;

{*********************************************************************
             SDLScene -  30.04.2003.

Author:      Stojkovic Igor
Contact:     jimyiigor@ptt.yu  (please report any bugs)
Description: TSDLScene - something like a form, container for other
             controls.

You may freely use this source code for your own projects.
If you have any suggestions or you made some modifications
please inform me.

What's new:
 - Added Movable property to determine if form can be moved
 - Used AlwaysUp property of TSDLStdControl to draw form up
   even when it is pressed - except it has an image
**********************************************************************}

interface

uses SDL, SDLUtils, SDLGUI, SDLDraw;

type
  TSDLScene = class(TSDLStdControl)
  private
    FDraging : Boolean;
    FOldPos : TPoint;
    FCaption: string;
  protected
    procedure SetZ(const Value: Integer); override;
    procedure DoDraw; override;
    procedure MouseMove(Modifier: TSDLMod;AX,AY : Word); override;
    procedure MouseUp(Button: Integer;Modifier: TSDLMod;AX,AY: Integer); override;
    procedure MouseDown(Button: Integer;Modifier: TSDLMod;AX,AY: Integer); override;
    function LoseFocus: Boolean; override;
  public
    Modal : Boolean;
    TitleBarHeight : Word;
    Movable: Boolean;
    constructor Create(AParent: TSDLObject); override;
    procedure Close;
    function SetFocus: Boolean; override;
    property Caption: string read FCaption write FCaption;
  end;

implementation

var SceneCount : Byte;

{ TSDLScene }

procedure TSDLScene.Close;
var i,t : Integer;
    tc : TSDLImageObject;
begin
  i := Gui.IndexOfObject(Self);
  t := Gui.Count-1;
  tc := nil;
  while (i<t)and not(tc is TSDLScene) do
  begin
    Inc(i);
    tc := Gui.Objects[i];
  end;
  if Assigned(tc)and(tc is TSDLScene) then
    tc.Z := tc.Z-1;
  Dec(SceneCount);
  Kill;
end;

constructor TSDLScene.Create(AParent: TSDLObject);
begin
  Inc(SceneCount);
  inherited;

  SetFocus;
  TitleBarHeight := 25;
  Movable := True;
  AlwaysUp := True;
end;

procedure TSDLScene.DoDraw;
var r : TSDL_Rect;
    c,ct : Cardinal;
begin
  inherited;
  if (Caption='')or(TitleBarHeight<5) then Exit;

  r := BoundsRect;
  r.h := TitleBarHeight;
  Font.TextRect(Gui.Surface,r,FCaption);

  if Assigned(Image) then Exit;

  if HasFocus then c := FocusColor
  else if not Enabled then c := DisabledColor
  else c := Color;

  ct := SDL_MapCardinal(Gui.Surface.Format,DarkColor(c));
  c := SDL_MapCardinal(Gui.Surface.Format,LightColor(c));
  SDL_DrawOrthoLine(Gui.Surface,True,X+BorderWidth,Y+TitleBarHeight,
                Width-BorderWidth shl 1,c);
  SDL_DrawOrthoLine(Gui.Surface,True,X+BorderWidth,Y+TitleBarHeight+1,
                Width-BorderWidth shl 1,ct);
end;

function TSDLScene.LoseFocus: Boolean;
begin
  if Modal then
    Result := False
  else
    Result := inherited LoseFocus;
end;

procedure TSDLScene.MouseDown(Button: Integer;Modifier: TSDLMod;AX,AY: Integer);
var r : TSDL_Rect;
begin
  inherited;
  if (Button<>1)or not Movable then Exit;
  r := BoundsRect;
  r.h := TitleBarHeight;
  FOldPos := SDL_Point(AX,AY);
  if PointInRect(FOldPos,r) then
    FDraging := True;
end;

procedure TSDLScene.MouseMove(Modifier: TSDLMod; AX, AY: Word);
begin
  inherited;
  if FDraging then
  begin
    X := X+(AX-FOldPos.X);
    Y := Y+(AY-FOldPos.Y);
    FOldPos.X := AX;
    FOldPos.Y := AY;
  end;
end;

procedure TSDLScene.MouseUp(Button: Integer;Modifier: TSDLMod; AX,AY: Integer);
begin
  inherited;
  FDraging := False;
end;

function TSDLScene.SetFocus: Boolean;
begin
  Result := inherited SetFocus;
  if Result then
    Z := SceneCount;
end;

procedure TSDLScene.SetZ(const Value: Integer);
var i,t,v : Integer;
    tc : TSDLImageObject;
begin
  v := Value;
  if v>=SceneCount then
    if HasFocus then
      v := SceneCount
    else
      v := SceneCount-1;

  i := Gui.IndexOfObject(Self);
  t := Gui.Count-1;
  tc := nil;
  while (i<t)and not(tc is TSDLScene) do
  begin
    Inc(i);
    tc := Gui.Objects[i];
  end;
  if Assigned(tc)and(tc is TSDLScene) then
    tc.Z := tc.Z-1;
  inherited SetZ(v);
end;

end.
